#include "SKeyView.h"

#include "skey.h"

#include <interface/View.h>
#include <interface/TextControl.h>
#include <interface/RadioButton.h>
#include <app/Clipboard.h>
#include <kernel/OS.h>

#include <ctype.h>

OTPView::OTPView(BRect frame)
		:BView(frame, NULL, B_FOLLOW_LEFT | B_FOLLOW_TOP,
			   B_WILL_DRAW | B_FRAME_EVENTS)
{
	output_sem = create_sem(1, "output");
	data[0] = '\0';
}

void OTPView::AttachedToWindow()
{
	BView::AttachedToWindow();
	GetFontInfo(&font);
}

void OTPView::Draw(BRect update_rect)
{
	BRect bounds = Bounds();

	FillRect(update_rect, B_SOLID_LOW);

	// Draw a spiffy shaded rectangle.
	SetHighColor(180, 180, 180);
	StrokeLine(BPoint(0, 0), BPoint(0, bounds.bottom));
	StrokeLine(BPoint(0, 0), BPoint(bounds.right, 0));
	SetHighColor(240, 240, 240);
	StrokeLine(BPoint(bounds.right, 0), BPoint(bounds.right, bounds.bottom));
	StrokeLine(BPoint(0, bounds.bottom), BPoint(bounds.right, bounds.bottom));
	SetHighColor(0, 0, 0);

	acquire_sem(output_sem);
	DrawString(data, BPoint(5, bounds.top+
						    (bounds.bottom+font.ascent)/2));
	release_sem(output_sem);
}

void OTPView::SetText(const char *string)
{
	if (!string || !*string)
		return;

	acquire_sem(output_sem);
	strncpy(data, string, sizeof(data));
	release_sem(output_sem);
	Draw(Bounds());
}

const char *OTPView::Text()
{
	return data;
}

SKeyView::SKeyView(BRect frame)
		 :BView(frame, "view", B_FOLLOW_ALL_SIDES,
				B_WILL_DRAW | B_FRAME_EVENTS)
{
}

void SKeyView::AttachedToWindow()
{
	BRect bounds = Bounds();

	SetViewColor(235, 235, 235);

	// Create all the sub-views here.
	SeedView = new BTextControl(BRect(5, 3, bounds.right-5, 20), NULL,
								SEED_PROMPT, NULL, NULL);
	AddChild(SeedView);
	SeedView->SetDivider(SeedView->StringWidth(SEED_PROMPT)+10);

	PWView = new BTextControl(BRect(5, 25, bounds.right-5, 42), NULL, KEY_PROMPT, NULL,
						   new BMessage(SKEY_COMP_KEY));
	AddChild(PWView);
	PWView->ChildAt(0)->SetFontName("Courier New");
	PWView->ChildAt(0)->SetFontSize(0);

	PWView->SetDivider(SeedView->StringWidth(KEY_PROMPT)+10);

	BRadioButton *radio_button = new BRadioButton(BRect(bounds.right-BUTTON_WIDTH-5, 50,
										  bounds.right-85, 100), NULL, "MD4",
									new BMessage(MD4_MSG));
	AddChild(radio_button);

	// Set the default value.
	radio_button->SetValue(B_CONTROL_ON);
	md4 = true;

	BRect frame = radio_button->Frame();
	frame.left = frame.right + 5; frame.right = bounds.right-5;
	radio_button = new BRadioButton(frame, NULL, "MD5", new BMessage(MD5_MSG));
	AddChild(radio_button);

	// Put a box around the password view.
	BBox *pw_box = new BBox(BRect(5, 50, 5+OTP_BOX_WIDTH, 50+OTP_BOX_HEIGHT));
	pw_box->SetLabel("S/Key Password");
	pw_box->SetViewColor(235, 235, 235);
	pw_box->SetLowColor(235, 235, 235);
	AddChild(pw_box);
	// Create the view for the One Time Password.
	font_info font;
	pw_box->GetFontInfo(&font);
	KeyView = new OTPView(BRect(5, font.ascent+font.descent, OTP_BOX_WIDTH-5,
								OTP_BOX_HEIGHT-5));
	pw_box->AddChild(KeyView);

	BButton *button = new BButton(BRect(bounds.right-BUTTON_WIDTH-5, 68,
								  bounds.right-5, 0), NULL, BUTTON_LABEL,
								  new BMessage(SKEY_COMP_KEY));
	AddChild(button);
}

void SKeyView::MessageReceived(BMessage *msg)
{
	switch (msg->what) {
	case MD4_MSG:
		md4 = true;
		break;
	case MD5_MSG:
		md4 = false;
		break;
	case COPY_MSG:
		be_clipboard->Lock();
		be_clipboard->Clear();
		be_clipboard->AddText(KeyView->Text());
		be_clipboard->Commit();
		be_clipboard->Unlock();
		break;
	case PASTE_MSG: {
		long nbytes;
		const char *clipboard_seed;
		char *seed;
	
		be_clipboard->Lock();
		clipboard_seed = be_clipboard->FindText(&nbytes);
		seed = (char *)malloc(nbytes);
		memcpy(seed, clipboard_seed, nbytes);
		be_clipboard->Unlock();

		SeedView->SetText(seed);
		Window()->Lock();
		SeedView->MakeFocus();
		Window()->Unlock();
	}; break;
	case SKEY_COMP_KEY: {
		const char *seed = SeedView->Text();
		const char *key = PWView->Text();
		char *otp = ComputeKey(seed, key, md4);
		KeyView->SetText(otp);
	}; break;
	default:
		BView::MessageReceived(msg);
	}
}

char *ComputeKey(const char *seed, const char *key, bool md4)
{
	if (!seed)
		return NULL;

	char *bogon, number[10], crunched[128];
	static char otp8[64];

	// Ignore everything before the first digit.
	for (; *seed && !isdigit(*seed); seed++);

	if (!*seed)
		return NULL;

	// Set the seed index to the first non-digit after that.
	long i;
	for (i=0; i < sizeof(number) && *seed && isdigit(*seed); seed++) {
		number[i] = *seed; i++;
	}
	number[i] = '\0';

	if (!*seed)
		return NULL;

	int iter = strtol(number, &bogon, NULL);

	// Skip the white space.
	for (; seed && *seed && isspace(*seed); seed++);

	if (md4) {
		md4_keycrunch(crunched, (char *)seed, (char *)key);
		for (; iter > 0; iter--)
			md4_f(crunched);
	} else {
		md5_keycrunch(crunched, (char *)seed, (char *)key);
		for (; iter > 0; iter--)
			md5_f(crunched);
	}

	return put8(otp8, crunched);
}

