#include "StatusView.h"

#include <interface/View.h>
#include <kernel/OS.h>

StatusView::StatusView(BRect frame, const char *name)
		   : BTextView(frame, name, frame, B_FOLLOW_LEFT | B_FOLLOW_TOP,
				   B_WILL_DRAW | B_FRAME_EVENTS)
{
	output_sem = create_sem(1, "output");
	stat_mess[0] = '\0';
}

void StatusView::AttachedToWindow()
{
	BView::AttachedToWindow();
	GetFontInfo(&font);
	SetHighColor(0, 0, 0);
	SetLowColor(220, 220, 220);

	// Don't auto-draw.
	SetViewColor(B_TRANSPARENT_32_BIT);
}

void StatusView::Draw(BRect update_rect)
{
	FillRect(update_rect, B_SOLID_LOW);

	// Draw the border.
	SetHighColor(150, 150, 150);
	StrokeLine(BPoint(update_rect.left, update_rect.top),
			   BPoint(update_rect.right, update_rect.top));
	StrokeLine(BPoint(update_rect.right, update_rect.top),
			   BPoint(update_rect.right, update_rect.bottom));
	SetHighColor(0, 0, 0);

	acquire_sem(output_sem);
	update_rect.left += 2;
	update_rect.top += font.ascent+2;
	DrawString(stat_mess, BPoint(update_rect.left, update_rect.top));
	release_sem(output_sem);
}

void StatusView::SetText(const char *string)
{
	acquire_sem(output_sem);
	strncpy(stat_mess, string, sizeof(stat_mess)-1);
	release_sem(output_sem);
}
